@@ -38,6 +38,10 @@
 
 #define PY_SOURCE_TEMPLATE PACKAGE_DATA_DIR"/class-templates/py-source.tpl"
 
+#define JS_SOURCE_TEMPLATE PACKAGE_DATA_DIR"/class-templates/js-source.tpl"
+
+#define VALA_SOURCE_TEMPLATE PACKAGE_DATA_DIR"/class-templates/vala-source.tpl"
+
 typedef struct _CgWindowPrivate CgWindowPrivate;
 struct _CgWindowPrivate
 {
@@ -45,12 +49,22 @@ struct _CgWindowPrivate
 	GtkWidget *window;
 	
 	CgElementEditor *editor_cc;
+
 	CgElementEditor *editor_go_members;
 	CgElementEditor *editor_go_properties;
 	CgElementEditor *editor_go_signals;
+
 	CgElementEditor *editor_py_methods;
 	CgElementEditor *editor_py_constvars;
-	
+
+	CgElementEditor *editor_js_methods;
+	CgElementEditor *editor_js_variables;
+	CgElementEditor *editor_js_imports;
+
+	CgElementEditor *editor_vala_methods;
+	CgElementEditor *editor_vala_properties;
+	CgElementEditor *editor_vala_signals;
+
 	CgValidator *validator;
 };
 
@@ -129,6 +143,29 @@ static const CgElementEditorFlags GO_SIGNAL_FLAGS[] =
 	{ NULL, NULL }
 };
 
+static const gchar *VALA_BOOLEAN_LIST[] =
+{
+	"Yes",
+	"No",
+	NULL
+};
+
+static const gchar *VALA_METHSIG_SCOPE_LIST[] =
+{
+	"public",
+	"private",
+	"protected",
+	NULL
+}
+;
+static const gchar *VALA_PROP_SCOPE_LIST[] =
+{
+	"public",
+	"protected",
+	"internal",
+	NULL
+};
+
 #if 0
 static void
 cg_window_browse_button_clicked_cb (GtkButton *button,
@@ -310,6 +347,35 @@ cg_window_validate_py (CgWindow *window)
 		GTK_ENTRY (gtk_builder_get_object (priv->bxml, "py_name")), NULL);
 }
 
+static void
+cg_window_validate_js (CgWindow *window)
+{
+	CgWindowPrivate *priv;
+	priv = CG_WINDOW_PRIVATE (window);
+
+	if (priv->validator != NULL) g_object_unref (G_OBJECT (priv->validator));
+
+	priv->validator = cg_validator_new (
+		GTK_WIDGET (gtk_builder_get_object (priv->bxml, "create_button")),
+		GTK_ENTRY (gtk_builder_get_object (priv->bxml, "js_name")),
+		GTK_ENTRY (gtk_builder_get_object (priv->bxml, "source_file")), NULL);
+}
+
+static void
+cg_window_validate_vala (CgWindow *window)
+{
+	CgWindowPrivate *priv;
+	priv = CG_WINDOW_PRIVATE (window);
+
+	if (priv->validator != NULL) g_object_unref (G_OBJECT (priv->validator));
+
+	priv->validator = cg_validator_new (
+		GTK_WIDGET (gtk_builder_get_object (priv->bxml, "create_button")),
+		GTK_ENTRY (gtk_builder_get_object (priv->bxml, "vala_name")),
+		GTK_ENTRY (gtk_builder_get_object (priv->bxml, "source_file")), NULL);
+}
+
+
 static void
 cg_window_header_file_entry_set_sensitive (gpointer user_data, gboolean sensitive)
 {
@@ -349,6 +415,14 @@ cg_window_top_notebook_switch_page_cb (G_GNUC_UNUSED GtkNotebook *notebook,
 		cg_window_header_file_entry_set_sensitive (user_data, FALSE);
 		cg_window_validate_py (window);
 		break;
+	case 3: /* JavaScript */
+		cg_window_header_file_entry_set_sensitive (user_data, FALSE);
+		cg_window_validate_js (window);
+		break;
+	case 4: /* Vala */
+		cg_window_header_file_entry_set_sensitive (user_data, FALSE);
+		cg_window_validate_vala (window);
+		break;
 	default:
 		g_assert_not_reached ();
 		break;
@@ -492,8 +566,9 @@ cg_window_go_name_changed_cb (GtkEntry *entry,
 }
 
 static void
-cg_window_py_name_changed_cb (GtkEntry *entry,
-			      gpointer user_data)
+cg_window_dynamic_name_changed_cb (GtkEntry *entry,
+				   gpointer user_data,
+				   const gchar *file_ending)
 {
 	CgWindow *window;
 	CgWindowPrivate *priv;
@@ -512,7 +587,7 @@ cg_window_py_name_changed_cb (GtkEntry *entry,
 	str_filebase = cg_window_class_name_to_file_name (
 		gtk_entry_get_text (GTK_ENTRY (entry)));
 
-	str_filesource = g_strconcat (str_filebase, ".py", NULL);
+	str_filesource = g_strconcat (str_filebase, file_ending, NULL);
 	g_free (str_filebase);
 	
 	gtk_entry_set_text (GTK_ENTRY (file_header), str_filesource);
@@ -521,6 +596,57 @@ cg_window_py_name_changed_cb (GtkEntry *entry,
 	g_free (str_filesource);
 }
 
+static void
+cg_window_py_name_changed_cb (GtkEntry *entry,
+			      gpointer user_data)
+{
+	cg_window_dynamic_name_changed_cb (entry, user_data, ".py");
+}
+
+static void
+cg_window_js_name_changed_cb (GtkEntry *entry,
+			      gpointer user_data)
+{
+	cg_window_dynamic_name_changed_cb (entry, user_data, ".js");
+}
+
+static void
+cg_window_js_is_subclass_changed_cb (GtkEntry *entry,
+				     gpointer user_data)
+{
+	CgWindow *window;
+	CgWindowPrivate *priv;
+
+	GtkWidget *is_subclass;
+	GtkWidget *entry_base_class;
+	GtkWidget *label_base_class;
+
+	window = CG_WINDOW (user_data);
+	priv = CG_WINDOW_PRIVATE (window);
+
+	is_subclass = GTK_WIDGET (gtk_builder_get_object (priv->bxml, "js_is_subclass"));
+	entry_base_class = GTK_WIDGET (gtk_builder_get_object (priv->bxml, "js_base"));
+	label_base_class = GTK_WIDGET (gtk_builder_get_object (priv->bxml, "lbl_js_base"));
+
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (is_subclass)) == TRUE)
+	{
+		gtk_editable_set_editable (GTK_EDITABLE (entry_base_class), TRUE);
+		gtk_widget_set_sensitive (label_base_class, TRUE);
+	}
+	else
+	{
+		gtk_editable_set_editable (GTK_EDITABLE (entry_base_class), FALSE);
+		gtk_widget_set_sensitive (label_base_class, FALSE);
+	}
+}
+
+static void
+cg_window_vala_name_changed_cb (GtkEntry *entry,
+			      gpointer user_data)
+{
+	cg_window_dynamic_name_changed_cb (entry, user_data, ".vala");
+}
+
 #if 0
 static void
 cg_window_associate_browse_button (GladeXML *xml,
@@ -621,6 +747,61 @@ cg_window_set_builder (CgWindow *window,
 		_("Name"), CG_ELEMENT_EDITOR_COLUMN_STRING,
 		_("Value"), CG_ELEMENT_EDITOR_COLUMN_STRING);
 
+	priv->editor_js_methods = cg_element_editor_new (
+		GTK_TREE_VIEW (gtk_builder_get_object (priv->bxml, "js_methods")),
+		GTK_BUTTON (gtk_builder_get_object (priv->bxml, "js_methods_add")),
+		GTK_BUTTON (gtk_builder_get_object (priv->bxml, "js_methods_remove")),
+		2,
+		_("Name"), CG_ELEMENT_EDITOR_COLUMN_STRING,
+		_("Arguments"), CG_ELEMENT_EDITOR_COLUMN_ARGUMENTS);
+
+	priv->editor_js_variables = cg_element_editor_new (
+		GTK_TREE_VIEW (gtk_builder_get_object (priv->bxml, "js_variables")),
+		GTK_BUTTON (gtk_builder_get_object (priv->bxml, "js_variables_add")),
+		GTK_BUTTON (gtk_builder_get_object (priv->bxml, "js_variables_remove")),
+		2,
+		_("Name"), CG_ELEMENT_EDITOR_COLUMN_STRING,
+		_("Value"), CG_ELEMENT_EDITOR_COLUMN_STRING);
+
+	priv->editor_js_imports = cg_element_editor_new (
+		GTK_TREE_VIEW (gtk_builder_get_object (priv->bxml, "js_imports")),
+		GTK_BUTTON (gtk_builder_get_object (priv->bxml, "js_imports_add")),
+		GTK_BUTTON (gtk_builder_get_object (priv->bxml, "js_imports_remove")),
+		2,
+		_("Name"), CG_ELEMENT_EDITOR_COLUMN_STRING,
+		_("Module"), CG_ELEMENT_EDITOR_COLUMN_STRING);
+
+	priv->editor_vala_methods = cg_element_editor_new (
+		GTK_TREE_VIEW (gtk_builder_get_object (priv->bxml, "vala_methods")),
+		GTK_BUTTON (gtk_builder_get_object (priv->bxml, "vala_methods_add")),
+		GTK_BUTTON (gtk_builder_get_object (priv->bxml, "vala_methods_remove")),
+		4,
+		_("Scope"), CG_ELEMENT_EDITOR_COLUMN_LIST, VALA_METHSIG_SCOPE_LIST,
+		_("Type"), CG_ELEMENT_EDITOR_COLUMN_STRING,
+		_("Name"), CG_ELEMENT_EDITOR_COLUMN_STRING,
+		_("Arguments"), CG_ELEMENT_EDITOR_COLUMN_ARGUMENTS);
+
+	priv->editor_vala_properties = cg_element_editor_new (
+		GTK_TREE_VIEW (gtk_builder_get_object (priv->bxml, "vala_properties")),
+		GTK_BUTTON (gtk_builder_get_object (priv->bxml, "vala_properties_add")),
+		GTK_BUTTON (gtk_builder_get_object (priv->bxml, "vala_properties_remove")),
+		7,
+		_("Scope"), CG_ELEMENT_EDITOR_COLUMN_LIST, VALA_PROP_SCOPE_LIST,
+		_("Type"), CG_ELEMENT_EDITOR_COLUMN_STRING,
+		_("Name"), CG_ELEMENT_EDITOR_COLUMN_STRING,
+		_("Automatic"), CG_ELEMENT_EDITOR_COLUMN_LIST, VALA_BOOLEAN_LIST,
+		_("Getter"), CG_ELEMENT_EDITOR_COLUMN_LIST, VALA_BOOLEAN_LIST,
+		_("Setter"), CG_ELEMENT_EDITOR_COLUMN_LIST, VALA_BOOLEAN_LIST,
+		_("Value"), CG_ELEMENT_EDITOR_COLUMN_STRING);
+
+	priv->editor_vala_signals = cg_element_editor_new (
+		GTK_TREE_VIEW (gtk_builder_get_object (priv->bxml, "vala_signals")),
+		GTK_BUTTON (gtk_builder_get_object (priv->bxml, "vala_signals_add")),
+		GTK_BUTTON (gtk_builder_get_object (priv->bxml, "vala_signals_remove")),
+		3,
+		_("Scope"), CG_ELEMENT_EDITOR_COLUMN_LIST, VALA_METHSIG_SCOPE_LIST,
+		_("Name"), CG_ELEMENT_EDITOR_COLUMN_STRING,
+		_("Arguments"), CG_ELEMENT_EDITOR_COLUMN_ARGUMENTS);
 
 	/* Active item property in glade cannot be set because no GtkTreeModel
 	 * is assigned. */
@@ -634,6 +815,14 @@ cg_window_set_builder (CgWindow *window,
 		G_OBJECT (gtk_builder_get_object (priv->bxml, "py_name")), "changed",
 		G_CALLBACK (cg_window_py_name_changed_cb), window);
 
+	g_signal_connect (
+		G_OBJECT (gtk_builder_get_object (priv->bxml, "vala_name")), "changed",
+		G_CALLBACK (cg_window_vala_name_changed_cb), window);
+
+	gtk_combo_box_set_active (
+		GTK_COMBO_BOX (gtk_builder_get_object (priv->bxml, "vala_class_scope")),
+		0);
+
 	/* This revalidates the appropriate validator */
 	g_signal_connect (
 		G_OBJECT (gtk_builder_get_object (priv->bxml, "top_notebook")),
@@ -648,6 +837,14 @@ cg_window_set_builder (CgWindow *window,
 		G_OBJECT (gtk_builder_get_object (priv->bxml, "cc_name")), "changed",
 		G_CALLBACK (cg_window_cc_name_changed_cb), window);
 
+	g_signal_connect (
+		G_OBJECT (gtk_builder_get_object (priv->bxml, "js_name")), "changed",
+		G_CALLBACK (cg_window_js_name_changed_cb), window);
+
+	g_signal_connect (
+		G_OBJECT (gtk_builder_get_object (priv->bxml, "js_is_subclass")), "toggled",
+		G_CALLBACK (cg_window_js_is_subclass_changed_cb), window);
+
 	g_signal_connect (
 		G_OBJECT (gtk_builder_get_object (priv->bxml, "add_project")), "toggled",
 		G_CALLBACK (cg_window_add_project_toggled_cb), window);
@@ -771,6 +968,72 @@ cg_window_py_constvars_transform_func (GHashTable *table,
 	cg_transform_string (table, "Value");
 }
 
+static void
+cg_window_js_methods_transform_func (GHashTable *table,
+                                     G_GNUC_UNUSED gpointer user_data)
+{
+	cg_transform_arguments (table, "Arguments", FALSE);
+}
+
+static void
+cg_window_js_variables_transform_func (GHashTable *table,
+                                     G_GNUC_UNUSED gpointer user_data)
+{
+	
+	cg_transform_string (table, "Name");
+}
+
+static void
+cg_window_js_imports_transform_func (GHashTable *table,
+                                     G_GNUC_UNUSED gpointer user_data)
+{
+	cg_transform_string (table, "Name");
+}
+
+static void
+vala_transform_scope_func (GHashTable *table,
+                           G_GNUC_UNUSED gpointer user_data)
+{
+	gchar *scope;
+
+	scope = g_hash_table_lookup (table, "Scope");
+	if (scope == NULL)
+	{
+		g_hash_table_insert (table, "Scope",
+				     g_strdup_printf("public"));
+	}
+}
+
+static void
+cg_window_vala_methods_transform_func (GHashTable *table,
+                                       G_GNUC_UNUSED gpointer user_data)
+{
+	cg_transform_string (table, "Name");
+	cg_transform_arguments (table, "Arguments", FALSE);
+
+	vala_transform_scope_func (table, user_data);
+}
+
+static void
+cg_window_vala_signals_transform_func (GHashTable *table,
+                                       G_GNUC_UNUSED gpointer user_data)
+{
+	cg_transform_string (table, "Name");
+	cg_transform_arguments (table, "Arguments", FALSE);
+
+	vala_transform_scope_func (table, user_data);
+}
+
+static void
+cg_window_vala_properties_transform_func (GHashTable *table,
+                                          G_GNUC_UNUSED gpointer user_data)
+{
+	cg_transform_string (table, "Name");
+	cg_transform_string (table, "Value");
+
+	vala_transform_scope_func (table, user_data);
+}
+
 #if 0
 static gboolean
 cg_window_scope_condition_func (const gchar **elements,
@@ -820,6 +1083,12 @@ cg_window_init (CgWindow *window)
 	priv->editor_go_signals = NULL;
 	priv->editor_py_methods = NULL;
 	priv->editor_py_constvars = NULL;
+	priv->editor_js_methods = NULL;
+	priv->editor_js_variables = NULL;
+	priv->editor_js_imports = NULL;
+	priv->editor_vala_methods = NULL;
+	priv->editor_vala_properties = NULL;
+	priv->editor_vala_signals = NULL;
 	
 	priv->validator = NULL;
 }
@@ -845,6 +1114,18 @@ cg_window_finalize (GObject *object)
 		g_object_unref (G_OBJECT (priv->editor_py_methods));
 	if (priv->editor_py_constvars != NULL)
 		g_object_unref (G_OBJECT (priv->editor_py_constvars));
+	if (priv->editor_js_methods != NULL)
+		g_object_unref (G_OBJECT (priv->editor_js_methods));
+	if (priv->editor_js_variables != NULL)
+		g_object_unref (G_OBJECT (priv->editor_js_variables));
+	if (priv->editor_js_imports != NULL)
+		g_object_unref (G_OBJECT (priv->editor_js_imports));
+	if (priv->editor_vala_methods != NULL)
+		g_object_unref (G_OBJECT (priv->editor_vala_methods));
+	if (priv->editor_vala_properties != NULL)
+		g_object_unref (G_OBJECT (priv->editor_vala_properties));
+	if (priv->editor_vala_signals != NULL)
+		g_object_unref (G_OBJECT (priv->editor_vala_signals));
 
 	if (priv->validator != NULL)
 		g_object_unref (G_OBJECT (priv->validator));
@@ -1119,6 +1400,47 @@ cg_window_create_value_heap (CgWindow *window)
 					      cg_window_py_constvars_transform_func,
 					      window, "Name", "Value");
 		break;
+	case 3: /* JavaScript */
+		cg_window_set_heap_value (window, values, G_TYPE_STRING,
+					  "ClassName", "js_name");
+		cg_window_set_heap_value (window, values, G_TYPE_STRING,
+					  "BaseClass", "js_base");
+		cg_window_set_heap_value (window, values, G_TYPE_STRING,
+					  "Initargs", "js_initargs");
+		cg_window_set_heap_value (window, values, G_TYPE_BOOLEAN,
+					  "Headings", "js_headings");
+		cg_element_editor_set_values (priv->editor_js_methods, "Methods", values,
+					      cg_window_js_methods_transform_func,
+                                              window, "Name", "Arguments");
+		cg_element_editor_set_values (priv->editor_js_variables, "Variables",
+					      values,
+					      cg_window_js_variables_transform_func,
+					      window, "Name", "Value");
+		cg_element_editor_set_values (priv->editor_js_imports, "Imports",
+					      values,
+					      cg_window_js_imports_transform_func,
+					      window, "Name", "Module");
+		break;
+	case 4: /* Vala */
+		cg_window_set_heap_value (window, values, G_TYPE_STRING,
+					  "ClassName", "vala_name");
+		cg_window_set_heap_value (window, values, G_TYPE_STRING,
+				  "BaseClass", "vala_base");
+		cg_window_set_heap_value (window, values, G_TYPE_STRING,
+		                          "ClassScope", "vala_class_scope");
+		cg_window_set_heap_value (window, values, G_TYPE_BOOLEAN,
+					  "Headings", "vala_headings");
+		cg_element_editor_set_values (priv->editor_vala_methods, "Methods", values,
+					      cg_window_vala_methods_transform_func,
+					      window, "Scope", "Type", "Name", "Arguments");
+		cg_element_editor_set_values (priv->editor_vala_properties, "Properties", values,
+					      cg_window_vala_properties_transform_func,
+					      window, "Scope", "Type", "Name", "Automatic",
+					      "Getter", "Setter", "Value");
+		cg_element_editor_set_values (priv->editor_vala_signals, "Signals", values,
+					      cg_window_vala_signals_transform_func,
+					      window, "Scope", "Name", "Arguments");
+		break;
 	default:
 		g_assert_not_reached ();
 		break;
@@ -1171,6 +1493,10 @@ cg_window_get_header_template (CgWindow *window)
 		return GO_HEADER_TEMPLATE;
 	case 2: /* Python */
 		return PY_SOURCE_TEMPLATE;
+	case 3: /* JavaScript */
+		return JS_SOURCE_TEMPLATE;
+	case 4: /* Vala */
+		return VALA_SOURCE_TEMPLATE;
 	default:
 		g_assert_not_reached ();
 		return NULL;
@@ -1210,6 +1536,10 @@ cg_window_get_source_template(CgWindow *window)
 		return GO_SOURCE_TEMPLATE;
 	case 2:
 		return PY_SOURCE_TEMPLATE;
+	case 3: /* JavaScript */
+		return JS_SOURCE_TEMPLATE;
+	case 4: /* Vala */
+		return VALA_SOURCE_TEMPLATE;
 	default:
 		g_assert_not_reached ();
 		return NULL;
