@@ -67,6 +67,19 @@ typedef struct _InspectDialog InspectDialog;
 #define VALUE_ENTRY "value_entry"
 #define AUTO_UPDATE_CHECK "auto_update_check"
 
+
+enum targets {
+	TARGET_STRING,
+	TARGET_URL
+};
+
+static const GtkTargetEntry drag_targets[] = {
+	{"application-x/anjuta", GTK_TARGET_SAME_APP, TARGET_STRING},
+	{ "STRING",        GTK_TARGET_SAME_APP, TARGET_STRING },
+	{ "text/plain",    GTK_TARGET_SAME_APP, TARGET_STRING },
+	{ "text/uri-list", GTK_TARGET_SAME_APP, TARGET_URL }
+};
+
 /* Private functions
  *---------------------------------------------------------------------------*/
 
@@ -111,7 +124,7 @@ debug_tree_inspect_evaluate_dialog (ExprWatch * ew, const gchar* expression)
 			/* Add in watch window */
 			new_expr = debug_tree_get_first (dlg.tree);
 
-			if (new_expr != NULL)
+			if ((new_expr != NULL) && (strlen(new_expr) != 0))
 			{
 		    	var.expression = new_expr;
 				debug_tree_add_watch (ew->debug_tree, &var, FALSE);
@@ -199,12 +212,6 @@ debug_tree_change_watch_dialog (ExprWatch *ew, GtkTreeIter* iter)
 #endif
 }
 
-static void
-on_program_stopped (ExprWatch *ew)
-{
-	debug_tree_update_all (ew->debug_tree);
-}
-
 static void
 on_program_exited (ExprWatch *ew)
 {
@@ -212,7 +219,6 @@ on_program_exited (ExprWatch *ew)
 
 	/* Disconnect to other debugger signal */
 	g_signal_handlers_disconnect_by_func (ew->plugin, G_CALLBACK (on_program_exited), ew);
-	g_signal_handlers_disconnect_by_func (ew->plugin, G_CALLBACK (on_program_stopped), ew);
 }
 
 static void
@@ -224,7 +230,6 @@ on_program_started (ExprWatch *ew)
 	
 	/* Connect to other debugger signal */
 	g_signal_connect_swapped (ew->plugin, "program-exited", G_CALLBACK (on_program_exited), ew);
-	g_signal_connect_swapped (ew->plugin, "program-stopped", G_CALLBACK (on_program_stopped), ew);
 }
 
 /* Menu call backs
@@ -246,6 +251,10 @@ on_debug_tree_inspect (GtkAction *action, gpointer user_data)
 	{
 		expression = ianjuta_editor_get_current_word (IANJUTA_EDITOR (te), NULL);
 	}
+	if (g_regex_match_simple("^\\s*$", expression,G_REGEX_MULTILINE | G_REGEX_DOLLAR_ENDONLY, G_REGEX_MATCH_ANCHORED))
+	{
+		expression = NULL;
+	}
 	
 	debug_tree_inspect_evaluate_dialog (ew, expression);
 	g_free (expression);
@@ -315,7 +324,7 @@ on_debug_tree_update_all_watch (GtkAction *action, gpointer user_data)
 {
 	ExprWatch * ew = (ExprWatch *)user_data;
 	
-	debug_tree_update_all (ew->debug_tree);
+	debug_tree_update_tree (ew->debug_tree);
 }
 
 static void
@@ -362,6 +371,31 @@ on_debug_tree_button_press (GtkWidget *widget, GdkEventButton *bevent, gpointer
 	return FALSE;
 }
 
+static void
+on_debug_tree_drag_data_received (GtkWidget        *widget,
+								  GdkDragContext   *context,
+								  gint              x,
+								  gint              y,
+								  GtkSelectionData *selection_data,
+								  guint             info,
+								  guint             timestamp,
+								  gpointer          user_data)
+{
+	const gchar* signal_data = (gchar*) gtk_selection_data_get_data (selection_data);
+	IAnjutaDebuggerVariableObject var = {NULL, NULL, NULL, NULL, FALSE, FALSE, FALSE, -1};
+
+	if (signal_data != NULL)
+	{
+		var.expression = signal_data;
+		debug_tree_add_watch (((ExprWatch *)user_data)->debug_tree, &var, FALSE);
+	}
+
+	gtk_drag_finish (context, FALSE, FALSE, timestamp);
+
+	return;
+}
+
+
 /* Actions table
  *---------------------------------------------------------------------------*/
 
@@ -369,7 +403,7 @@ static GtkActionEntry actions_watch[] = {
     {
 		"ActionDmaInspect",                      /* Action name */
 		GTK_STOCK_DIALOG_INFO,                   /* Stock icon, if any */
-		N_("Ins_pect/Evaluate..."),              /* Display label */
+		N_("Ins_pect/Evaluate…"),              /* Display label */
 		NULL,                                    /* short-cut */
 		N_("Inspect or evaluate an expression or variable"), /* Tooltip */
 		G_CALLBACK (on_debug_tree_inspect) /* action callback */
@@ -377,7 +411,7 @@ static GtkActionEntry actions_watch[] = {
 	{
 		"ActionDmaAddWatch",                      
 		NULL,                                     
-		N_("Add Watch..."),                      
+		N_("Add Watch…"),                      
 		NULL,                                    
 		NULL,                                     
 		G_CALLBACK (on_debug_tree_add_watch)     
@@ -465,7 +499,10 @@ create_expr_watch_gui (ExprWatch * ew)
 											G_N_ELEMENTS (toggle_watch),
 											GETTEXT_PACKAGE, TRUE, ew);
 	g_signal_connect (debug_tree_get_tree_widget (ew->debug_tree), "button-press-event", G_CALLBACK (on_debug_tree_button_press), ew);  
-	
+
+	gtk_drag_dest_set(debug_tree_get_tree_widget (ew->debug_tree), GTK_DEST_DEFAULT_ALL, drag_targets, sizeof (drag_targets) / sizeof (drag_targets[0]), GDK_ACTION_COPY|GDK_ACTION_MOVE|GDK_ACTION_LINK);
+	g_signal_connect (debug_tree_get_tree_widget (ew->debug_tree), "drag_data_received", G_CALLBACK (on_debug_tree_drag_data_received), ew);
+
 	gtk_widget_show_all (ew->scrolledwindow);
 }
 
