@@ -19,7 +19,7 @@
 #include "stdio.h"
 
 #include "js-node.h"
-#include "y.tab.h"
+#include "js-parser-y-tab.h"
 #include "lex.yy.h"
 
 typedef struct _JSNodePrivate JSNodePrivate;
